import funciones_comunes as fc
import pandas as pd
import os
import re
import logging
from manejoArchivos import select_file


def complete_emptys(tipo_PLC, archivo_maestro, target_lang_code, second_lang_code):
    if not os.path.exists(archivo_maestro):
        print("El archivo maestro no existe.")
        return

    master_col = target_lang_code
    second_col = second_lang_code

    df_maestro = pd.read_excel(archivo_maestro)

    # Configurar el logger
    logger = fc.configurar_logger()

    # Iterar sobre las filas del archivo de traducción para actualizar el maestro
    for index, fila in df_maestro.iterrows():
        clave = fila[df_maestro.columns[0]]
        if fila[master_col] == "" or pd.isnull(fila[master_col]):
            if pd.notnull(fila[second_col]) and fila[second_col] != "":
                df_maestro.loc[
                    df_maestro[df_maestro.columns[0]] == clave, master_col
                ] = fila[second_col]
                logger.info(
                    f'Fila {index}, Columna {master_col}: " actualizado a "{fila[second_col]}"'
                )
            else:
                df_maestro.loc[
                    df_maestro[df_maestro.columns[0]] == clave, master_col
                ] = fc.compactar_celda_traducida(tipo_PLC, fila[df_maestro.columns[0]])
                logger.info(
                    f'Fila {index}, Columna {master_col}: " actualizado a "{fila[df_maestro.columns[0]]}"'
                )

    # Guardar el archivo maestro actualizado
    fc.save_dataframe_with_retries(
        df_maestro, output_path=archivo_maestro
    )
    print(
        f"Traducciones importadas y archivo maestro actualizado: {archivo_maestro}."
    )

def run(tipo_PLC, codigo_columna_maestra, seleccion_idioma, seleccion_idioma_secundario):
    archivo_maestro = f".\\data\\1_hmi_master_translates_{tipo_PLC}.xlsx"

    if seleccion_idioma not in fc.IDIOMAS:
        print("Selección inválida.")
        exit

    if seleccion_idioma_secundario not in fc.IDIOMAS:
        print("Selección inválida.")
        exit

    _, target_lang_code = fc.IDIOMAS[seleccion_idioma]
    _, second_lang_code = fc.IDIOMAS[seleccion_idioma_secundario]
    complete_emptys(tipo_PLC, archivo_maestro, target_lang_code, second_lang_code)

if __name__ == "__main__":
    tipo_PLC = "siemens"
    codigo_columna_maestra = "it-IT"
    fc.mostrar_idiomas()
    seleccion_idioma = int(input("Introduce el número del idioma de destino: "))
    if seleccion_idioma not in fc.IDIOMAS:
        print("Selección inválida.")
        exit
    seleccion_idioma_secundario = int(
        input(
            "Introduce el número del idioma de secundario para copiar desde en caso de vacios: "
        )
    )
    if seleccion_idioma_secundario not in fc.IDIOMAS:
        print("Selección inválida.")
        exit
    run(tipo_PLC, codigo_columna_maestra, seleccion_idioma, seleccion_idioma_secundario)

